'''
3D Custom Cube example show how to create custom scene node
CubeSceneNode(parent_node, scene_manager, x, y, z, color, ptype, material)
'''

driver_type = 5
full_screen = False
stencil_buffer = False
vsync = False

run_app = True

from video_choice_dialog import has_pywingui
if has_pywingui:
	from video_choice_dialog import ChoiceDialog, IDOK, IDCANCEL
	dialog = ChoiceDialog()
	dialog.driver_type = driver_type
	dialog.full_screen = full_screen
	dialog.stencil_buffer = stencil_buffer
	dialog.vsync = vsync
	dialogResult = dialog.DoModal()
	if dialogResult == IDOK:
		driver_type = dialog.driver_type
		full_screen = dialog.full_screen
		stencil_buffer = dialog.stencil_buffer
		vsync = dialog.vsync
	elif dialogResult == IDCANCEL:
		run_app = False

if run_app:
	from pyirrlicht import *

	if not driver_type:
		driver_type = EDT_OPENGL

	class CubeSceneNode(CustomSceneNode):
		def __init__(self, *args, **kwargs):
			self.color = kwargs.pop('color', SColor(255,0,255,0))
			x = kwargs.pop('x', 1.0)
			y = kwargs.pop('y', 1.0)
			z = kwargs.pop('z', 1.0)
			self.ptype = kwargs.pop('ptype', EPT_POLYGON)# also can be EPT_LINE_LOOP
			self.Material = kwargs.pop('material', 0)
			if not self.Material:
				self.Material = SMaterial()
				self.Material.Wireframe = False
				self.Material.Lighting = False
				self.Material.BackfaceCulling = False

			self.ver_count = 16
			self.Vertices = S3DVertex(self.ver_count)
			normal = vector3df(0.0,0.0,0.0)
			tcoords = vector2df(0.0, 0.0)

			self.Vertices[0] = S3DVertex(vector3df(0.0,0.0,0.0), normal, self.color, tcoords)
			self.Vertices[1] = S3DVertex(vector3df(x,0.0,0.0), normal, self.color, tcoords)
			self.Vertices[2] = S3DVertex(vector3df(x,0.0,z), normal, self.color, tcoords)
			self.Vertices[3] = S3DVertex(vector3df(0.0,0.0,z), normal, self.color, tcoords)
			self.Vertices[4] = S3DVertex(vector3df(0.0,0.0,0.0), normal, self.color, tcoords)
			self.Vertices[5] = S3DVertex(vector3df(0.0,y,0.0), normal, self.color, tcoords)
			self.Vertices[6] = S3DVertex(vector3df(x,y,0.0), normal, self.color, tcoords)
			self.Vertices[7] = S3DVertex(vector3df(x,0.0,0.0), normal, self.color, tcoords)
			self.Vertices[8] = S3DVertex(vector3df(x,0.0,z), normal, self.color, tcoords)
			self.Vertices[9] = S3DVertex(vector3df(x,y,z), normal, self.color, tcoords)
			self.Vertices[10] = S3DVertex(vector3df(0.0,y,z), normal, self.color, tcoords)
			self.Vertices[11] = S3DVertex(vector3df(0.0,y,0.0), normal, self.color, tcoords)
			self.Vertices[12] = S3DVertex(vector3df(x,y,0.0), normal, self.color, tcoords)
			self.Vertices[13] = S3DVertex(vector3df(x,y,z), normal, self.color, tcoords)
			self.Vertices[14] = S3DVertex(vector3df(0.0,y,z), normal, self.color, tcoords)
			self.Vertices[15] = S3DVertex(vector3df(0.0,0.0,z), normal, self.color, tcoords)

			self.Box = aabbox3df()
			self.Box.reset(self.Vertices[0].Pos)
			for i in range(1, self.ver_count):
				self.Box.addInternalPoint(self.Vertices[i].Pos)

			CustomSceneNode.__init__(self, *args, **kwargs)

			self.scene_manager = self.getSceneManager()
			self.video_driver = self.scene_manager.getVideoDriver()
			self.iType = EIT_32BIT
			if self.video_driver.getDriverType() == EDT_SOFTWARE:
				self.iType = EIT_16BIT
			if self.video_driver.getDriverType() == EDT_BURNINGSVIDEO:
				self.render = self.safe_render
				CustomSceneNode.__init__(self, *args, **kwargs)

		def getTypeS3DVertex(self):
			return 0

		def OnRegisterSceneNode(self):
			if self.isVisible():
				self.scene_manager.registerNodeForRendering(self)

		def render(self):
			list_indices = []
			for i in range(0, self.ver_count):
				list_indices.append(i)
			indices = (ctypes.c_int * self.ver_count)(*list_indices)
			self.video_driver.setMaterial(self.Material)
			self.video_driver.setTransform(ETS_WORLD, self.getAbsoluteTransformation())
			self.video_driver.drawVertexPrimitiveList(self.Vertices, self.ver_count, indices, self.ver_count, EVT_STANDARD, self.ptype, self.iType)

		def safe_render(self):
			list_indices = []
			for i in range(0, self.ver_count):
				list_indices.append(i)
			indices = (ctypes.c_int * self.ver_count)(*list_indices)
			self.video_driver.setMaterial(self.Material)
			self.video_driver.setTransform(ETS_WORLD, self.getAbsoluteTransformation())
			try:
				self.video_driver.drawVertexPrimitiveList(self.Vertices, self.ver_count, indices, self.ver_count, EVT_STANDARD, self.ptype, self.iType)
			except:
				pass

		def getBoundingBox(self):
			return self.Box.c_pointer

		def getMaterialCount(self):
			return 1

		def getMaterial(self, num):
			return self.Material.c_pointer

	device = createDevice(driver_type, dimension2du(640, 480), 32, full_screen, stencil_buffer, vsync)
	if device:
		window_caption = 'Custom Cube Scene Node - Irrlicht Engine Demo'
		device.setResizable(True)
		driver = device.getVideoDriver()
		scene_manager = device.getSceneManager()
		camera = scene_manager.addCameraSceneNodeMaya(distance = 300.0)
		camera.setTarget(vector3df(0,50,0))
		base = scene_manager.addEmptySceneNode()
		nodes = (CubeSceneNode(base, scene_manager, x = 10.0, y = 5.0, z = 15.0, color = SColor(255,255,0,0)),
			CubeSceneNode(base, scene_manager, x = 10.0, y = 50.0, z = 30.0, color = SColor(255,0,255,0)),
			CubeSceneNode(base, scene_manager, x = 20.0, y = 20.0, z = 20.0, color = SColor(255,0,0,255), ptype = EPT_LINE_LOOP),
			CubeSceneNode(base, scene_manager, x = 30.0, y = 5.0, z = 15.0, color = SColor(255,255,255,0)))
		nodes[1].setPosition(vector3df(100.0, 0.0, 0.0))
		nodes[2].setPosition(vector3df(100.0, 0.0, 50.0))
		nodes[3].setPosition(vector3df(0.0, 0.0, 50.0))
		anim_pos = vector3df(0.0, 0.0, 1.0)
		animators = (scene_manager.createRotationAnimator(anim_pos),
			scene_manager.createRotationAnimator(anim_pos),
			scene_manager.createRotationAnimator(anim_pos),
			scene_manager.createRotationAnimator(anim_pos))
		i = 0
		for node in nodes:
			node.setMaterialFlag(EMF_LIGHTING, False)
			node.addAnimator(animators[i])
			animators[i].drop()
			node.drop()
			i = i + 1
		base.setRotation(vector3df(0.0, 90.0, 0.0))
		base.addAnimator(scene_manager.createRotationAnimator(vector3df(0.0, 0.06, 0.0)))
		base.addAnimator(scene_manager.createFlyCircleAnimator(radius = 200.0, speed = 0.001))
		frames = 0
		scene_color = SColor(0, 100, 100, 100)
		while device.run():
			if device.isWindowActive():
				frames += 1
				if driver.beginScene(True, True, scene_color):
					scene_manager.drawAll()
					driver.endScene()
			else:
				device.yield_self()
			if frames == 100:
				device.setWindowCaption("%s [%s] FPS: %d" % (window_caption, driver.getName(), driver.getFPS()))
				frames=0
		device.drop()
	else:
		print('ERROR createDevice')
