'Text3D - Irrlicht Engine Demo'

from pyirrlicht import BUILD_WITH_3D_TEXT
if not BUILD_WITH_3D_TEXT:
	print('Sorry, irrlicht_c library compiled without support 3D text.\nSee _COMPILE_WITH_3D_TEXT_ flag from irrlicht_c.h header file.')
else:
	driver_type = 5
	full_screen = False
	stencil_buffer = False
	vsync = False

	run_app = True

	from video_choice_dialog import has_pywingui
	if has_pywingui:
		from video_choice_dialog import ChoiceDialog, IDOK, IDCANCEL
		dialog = ChoiceDialog()
		dialog.driver_type = driver_type
		dialog.full_screen = full_screen
		dialog.stencil_buffer = stencil_buffer
		dialog.vsync = vsync
		dialogResult = dialog.DoModal()
		if dialogResult == IDOK:
			driver_type = dialog.driver_type
			full_screen = dialog.full_screen
			stencil_buffer = dialog.stencil_buffer
			vsync = dialog.vsync
		elif dialogResult == IDCANCEL:
			run_app = False

	if run_app:
		from pyirrlicht import *
		if not driver_type:
			#driver_type = EDT_SOFTWARE
			#driver_type = EDT_DIRECT3D9
			driver_type = EDT_OPENGL
		window_size = dimension2du(640, 480)
		device = createDevice(driver_type, window_size, 16, full_screen, stencil_buffer, vsync, 0)

		if device:
			window_caption = __doc__
			device.setResizable(True)
			driver = device.getVideoDriver()
			scene_manager = device.getSceneManager()
			guienv = device.getGUIEnvironment()
			static_text = guienv.addStaticText(window_caption, recti(10, 10, 300, 22), True)
			#~ root_scene_node = scene_manager.getRootSceneNode()
			root_scene_node = scene_manager.addEmptySceneNode()

			# set plane 3d text
			t3d = Text3DPlane(root_scene_node, scene_manager)
			t3d.getMaterial(0).EmissiveColor = SColor(255, 100, 0, 100)
			result_set_text = t3d.setText('Plane' + __doc__)
			t3d.setPosition(vector3df(300, 900, 1800))
			t3d.setRotation(vector3df(0, 90, 0))
			t3d.setIsDebugObject()

			# set simple 3d text
			text3d = Text3DSimple(root_scene_node, scene_manager)
			text3d.getMaterial(0).EmissiveColor = SColor(255, 100, 0, 100)
			result_set_text = text3d.setText('Simple' + __doc__)
			text3d.setPosition(vector3df(300, 100, 1800))
			text3d.setRotation(vector3df(0, 90, 0))
			text3d.setIsDebugObject()

			# set camera
			camera = scene_manager.addCameraSceneNodeFPS(moveSpeed = 1.0)
			camera.setFarValue(9000.0)
			camera.setPosition(vector3df(-1300, 500, -1300))

			device.getCursorControl().setVisible(False)

			scolor = SColor(255, 100, 100, 100)
			lastFPS = -1
			angle = 0
			while device.run():
				if device.isWindowActive():
					if driver.beginScene(True, True, scolor):
						scene_manager.drawAll()
						guienv.drawAll()
						driver.endScene()
					#~ device.sleep(10)
					fps = driver.getFPS()
					if lastFPS != fps:
						text = '%s [%s] FPS:%d' % (window_caption, driver.getName(), fps)
						device.setWindowCaption(text)
						static_text.setText(text)
						lastFPS = fps
				else:
					device._yield()
			device.closeDevice()
		else:
			print('ERROR createDevice')
